﻿/*	File			TimeServiceClient_GUI_Phase2_Refactored.cs
	Purpose			Network Time Protocol (NTP) Client
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			February 2015
*/

using System;
using System.Net;
using System.Net.Sockets;
using System.Windows.Forms;

namespace TimeServiceClient
{
    public partial class TimeServiceClient_GUI_Phase2_Refactored : Form
    {
        NTP_Client m_NTP_Client;
        NTP_Client.NTP_Timestamp_Data m_NTP_Timestamp;
        System.Windows.Forms.Timer m_Timer_SendNTPRequests;
        Int32 m_iNumRequestsSent;
        Int32 m_iNumResponsesReceived;

        public TimeServiceClient_GUI_Phase2_Refactored()
        {
            bool bSocketOpenSuccess = false;
            m_NTP_Client = new NTP_Client(ref bSocketOpenSuccess);
            if (false == bSocketOpenSuccess)
            {
                MessageBox.Show("Error creating socket", "NTP client");
                CloseSocketAndExit();
            }

            InitializeComponent();
            Populate_NTP_Server_List();
 	        m_NTP_Client.Set_ClientStarted_Flag(false);
	        m_iNumRequestsSent = 0;
	        m_iNumResponsesReceived = 0;
            UpdateStatisticsDisplay();
            InitialiseControls();
        }

        void InitialiseControls()
        {
            IP_1_textBox.Enabled = false;
            IP_2_textBox.Enabled = false;
            IP_3_textBox.Enabled = false;
            IP_4_textBox.Enabled = false;
            URL_textBox.Enabled = false;
            Port_textBox.Enabled = false;
            ServerURL_listBox.Enabled = true;
            ServerLocation_listBox.Enabled = false;
            UNIX_Time_textBox.Enabled = false;
            UTC_Time_textBox.Enabled = false;
            Num_NTP_Requests_textBox.Enabled = false;
            NUM_NTP_Responses_textBox.Enabled = false;
            Initialise_ServerURL_listBox(); // Selects first item in list boxes, by default
         }

        void Start_button_Click(object sender, EventArgs e)
        {
            if (false == m_NTP_Client.Get_ClientStarted_Flag())
	        {
                Start_button.Text = "STOP NTP requests";
                ServerURL_listBox.Enabled = false;
                m_iNumRequestsSent = 0;
		        m_iNumResponsesReceived = 0;
		        UpdateStatisticsDisplay();
		        StartPeriodic_NTP_requests();
                m_NTP_Client.Set_ClientStarted_Flag(true);
            }
	        else
	        {
		        Start_button.Text = "START NTP requests";
                ServerURL_listBox.Enabled = true;
                m_NTP_Client.Set_ClientStarted_Flag(false);
		        StopPeriodic_NTP_requests();
	        }
        }

        void StartPeriodic_NTP_requests()
        {
            Start_Timer_SendNTPRequests();  // Subsequent NTP requests are made every 10 seconds
        }

        void StopPeriodic_NTP_requests()
        {
            StopTimer();
        }
        
        void Start_Timer_SendNTPRequests()
        {
	        // Do not make NTP requests faster than once every 4 seconds or NIST will treat it as a DoS attack
	        // This really happens (for example try sending at 200ms intervals :- refused after 22 requests)
            m_Timer_SendNTPRequests = new System.Windows.Forms.Timer();
            m_Timer_SendNTPRequests.Interval = 5000; // 5 second period for send NTP request
            m_Timer_SendNTPRequests.Tick += OnTimedEvent_Timer_SendNTPRequests;
            m_Timer_SendNTPRequests.Enabled = true;
        }

        void OnTimedEvent_Timer_SendNTPRequests(Object myObject, EventArgs myEventArgs)
        {
            Get_NTP_Timestamp();
        }

        void Get_NTP_Timestamp()
        {
            NTP_Client.NTP_Client_ResultCode Result = m_NTP_Client.Get_NTP_Timestamp(ref m_NTP_Timestamp);
            switch (Result)
            {
                case NTP_Client.NTP_Client_ResultCode.NTP_Success:
                    m_iNumRequestsSent++;
                    m_iNumResponsesReceived++;
                    UNIX_Time_textBox.Text = m_NTP_Timestamp.lUnixTime.ToString();
                    string sUTC_Time = string.Format("{0:00}:{1:00}:{2:00}", m_NTP_Timestamp.iHour, m_NTP_Timestamp.iMinute, m_NTP_Timestamp.iSecond);
                    UTC_Time_textBox.Text = sUTC_Time;
                    // NTP does not indicate information specifcally related to local time zones such as BST or other daylight savings time offsets
                    UpdateStatisticsDisplay();
                    break;
                case NTP_Client.NTP_Client_ResultCode.NTP_ServerAddressNotSet:
                    break;
                case NTP_Client.NTP_Client_ResultCode.NTP_SendFailed:
                    break;
                case NTP_Client.NTP_Client_ResultCode.NTP_ReceiveFailed:
                    m_iNumRequestsSent++;
                    UpdateStatisticsDisplay();
                    break;
            }
        }

        void SetUp_TimeService_AddressStruct(string sURL)
        {
            string sIP_Address1 = "";
            string sIP_Address2 = "";
            string sIP_Address3 = "";
            string sIP_Address4 = "";
            string sPort = "";
            
            m_NTP_Client.SetUp_TimeService_AddressStruct(sURL, ref sIP_Address1, 
                ref sIP_Address2, ref sIP_Address3, ref sIP_Address4, ref sPort);
            IP_1_textBox.Text = sIP_Address1;
            IP_2_textBox.Text = sIP_Address2;
            IP_3_textBox.Text = sIP_Address3;
            IP_4_textBox.Text = sIP_Address4;
            Port_textBox.Text = sPort;
        }

        void Populate_NTP_Server_List()
        {
            ServerURL_listBox.Items.Add("time.nist.gov");
            ServerLocation_listBox.Items.Add("NIST round robin load equalisation");
            ServerURL_listBox.Items.Add("time.windows.com");
            ServerLocation_listBox.Items.Add("Windows Time service");
            ServerURL_listBox.Items.Add("nist1-atl.ustiming.org");
	        ServerLocation_listBox.Items.Add("Atlanta, Georgia");
	        ServerURL_listBox.Items.Add("wolfnisttime.com");
	        ServerLocation_listBox.Items.Add("Birmingham, Alabama");
	        ServerURL_listBox.Items.Add("nist1-chi.ustiming.org");
	        ServerLocation_listBox.Items.Add("Chicago, Illinois");
	        ServerURL_listBox.Items.Add("nist1-lnk.binary.net");
	        ServerLocation_listBox.Items.Add("Lincoln, Nebraska");
	        ServerURL_listBox.Items.Add("time-a.timefreq.bldrdoc.gov");
	        ServerLocation_listBox.Items.Add("NIST, Boulder, Colorado");
	        ServerURL_listBox.Items.Add("ntp-nist.ldsbc.edu");
	        ServerLocation_listBox.Items.Add("LDSBC, Salt Lake City, Utah");
	        ServerURL_listBox.Items.Add("nist1-lv.ustiming.org");
	        ServerLocation_listBox.Items.Add("Las Vegas, Nevada");  
	        ServerURL_listBox.Items.Add("nist1-la.ustiming.org");
	        ServerLocation_listBox.Items.Add("Los Angeles, California");
            ServerURL_listBox.Items.Add("nist1-ny.ustiming.org");
            ServerLocation_listBox.Items.Add("New York City, NY");
            ServerURL_listBox.Items.Add("nist1-nj.ustiming.org");
            ServerLocation_listBox.Items.Add("Bridgewater, NJ");
        }

        private void Initialise_ServerURL_listBox()
        {
            ServerURL_listBox.SelectedIndex = 0;
            ServerLocation_listBox.SelectedIndex = 0;
            Get_ServerURL_listBox_Selection();
        }

        private void Get_ServerURL_listBox_Selection()
        {
            string sSelectedURL = ServerURL_listBox.SelectedItem.ToString();
            URL_textBox.Text = sSelectedURL;
            SetUp_TimeService_AddressStruct(sSelectedURL);
        }

        private void ServerURL_listBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            int iSelectionIndex = ServerURL_listBox.SelectedIndex;
            ServerLocation_listBox.SelectedIndex = iSelectionIndex;
            Get_ServerURL_listBox_Selection();
            UNIX_Time_textBox.Text = "";
            UTC_Time_textBox.Text = "";
            m_iNumRequestsSent = 0;
            m_iNumResponsesReceived = 0;
            UpdateStatisticsDisplay();
        }

        void UpdateStatisticsDisplay()
        {
	        Num_NTP_Requests_textBox.Text = m_iNumRequestsSent.ToString();
            NUM_NTP_Responses_textBox.Text = m_iNumResponsesReceived.ToString();
        }

        private void Done_Button_Click(object sender, EventArgs e)
        {
            CloseSocketAndExit();
        }

        void StopTimer()
        {
            if (null != m_Timer_SendNTPRequests)
            {
                m_Timer_SendNTPRequests.Enabled = false;
            }
        }

        void CloseSocketAndExit()
        {
            StopTimer();
            m_NTP_Client.CloseSocket();
            System.Environment.Exit(-1);
        }
    }
}
